diff --git a/channels/chan_iax2.c b/channels/chan_iax2.c
index 8d3e7f1..4a09ee2 100644
--- a/channels/chan_iax2.c
+++ b/channels/chan_iax2.c
@@ -2056,6 +2056,9 @@ static int iax2_getpeername(struct ast_sockaddr addr, char *host, int len)
  * iax2_destroy_helper() is called. */
 static void iax2_destroy_helper(struct chan_iax2_pvt *pvt)
 {
+	int pingid = pvt->pingid;
+	int lagid = pvt->lagid;
+
 	/* Decrement AUTHREQ count if needed */
 	if (ast_test_flag64(pvt, IAX_MAXAUTHREQ)) {
 		struct iax2_user *user;
@@ -2069,10 +2072,18 @@ static void iax2_destroy_helper(struct chan_iax2_pvt *pvt)
 		ast_clear_flag64(pvt, IAX_MAXAUTHREQ);
 	}
 	/* No more pings or lagrq's */
-	AST_SCHED_DEL_SPINLOCK(sched, pvt->pingid, &iaxsl[pvt->callno]);
 	pvt->pingid = DONT_RESCHEDULE;
-	AST_SCHED_DEL_SPINLOCK(sched, pvt->lagid, &iaxsl[pvt->callno]);
 	pvt->lagid = DONT_RESCHEDULE;
+
+	/* Unlock callno to prevent dead lock while deleting ping or lag if they are already in running state started. */
+	ast_mutex_unlock(&iaxsl[pvt->callno]);
+
+	/* No more pings or lagrq's: Delete them from scheduler. */
+	AST_SCHED_DEL(sched, pingid);
+	AST_SCHED_DEL(sched, lagid);
+
+	ast_mutex_lock(&iaxsl[pvt->callno]);
+
 	AST_SCHED_DEL(sched, pvt->autoid);
 	AST_SCHED_DEL(sched, pvt->authid);
 	AST_SCHED_DEL(sched, pvt->initid);
diff --git a/main/sched.c b/main/sched.c
index 911143c..a6713b5 100644
--- a/main/sched.c
+++ b/main/sched.c
@@ -492,12 +492,26 @@ int _ast_sched_del(struct ast_sched_context *con, int id, const char *file, int
 		}
 		sched_release(con, s);
 	} else if (con->currently_executing && (id == con->currently_executing->id)) {
+		int ret;
+		struct timespec ts = {0};
+		struct timeval tv;
+		const struct timeval sleeptv = {.tv_sec= 0, .tv_usec= 1000};
+
 		s = con->currently_executing;
 		s->deleted = 1;
+
 		/* Wait for executing task to complete so that caller of ast_sched_del() does not
 		 * free memory out from under the task.
 		 */
-		ast_cond_wait(&s->cond, &con->lock);
+		do {
+			ast_log(LOG_WARNING, "Waiting for sched task %d to finish to delete it.\n", id);
+			tv = ast_tvadd(ast_tvnow(), sleeptv);
+			ts.tv_sec = tv.tv_sec;
+			ts.tv_nsec = tv.tv_usec * 1000;
+			/* Wait for 1ms before trying again. This is ONLY used to print warning that scheduler is stuck 
+			   until this task is finished. */
+			ret = ast_cond_timedwait(&s->cond, &con->lock, &ts);
+		} while(ret != 0);
 		/* Do not sched_release() here because ast_sched_runq() will do it */
 	}
 
